<?php
/*##################################################
 *		       BlogsDisplayBlogsController.class.php
 *                            -------------------
 *   begin                : April 03, 2013
 *   copyright            : (C) 2013 Patrick DUBEAU
 *   email                : daaxwizeman@gmail.com
 *
 *
 ###################################################
 *		Module Articles : Dupliqu et modifi.
 *--------------------------------------------------
 * Date: 31.05.2016
 * Par: Swan | EDN
 * Site: http://easy-design.net
 * Contact : contact@easy-design.net
 *
 ###################################################
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 ###################################################*/

class BlogsDisplayBlogsController extends ModuleController
{
	private $lang;
	private $tpl;
	private $blog;
	private $category;
	
	public function execute(HTTPRequestCustom $request)
	{
		$this->check_authorizations();
		
		$this->init();
		
		$this->check_pending_blog();
		
		$this->build_view($request);
		
		return $this->generate_response();
	}
	
	private function init()
	{
		$this->lang = LangLoader::get('common', 'blogs');
		$this->tpl = new FileTemplate('blogs/BlogsDisplayBlogsController.tpl');
		$this->tpl->add_lang($this->lang);
	}
	
	private function get_blog()
	{
		if ($this->blog === null)
		{
			$id = AppContext::get_request()->get_getint('id', 0);
			if (!empty($id))
			{
				try 
				{
					$this->blog = BlogsService::get_blog('WHERE blogs.id=:id', array('id' => $id));
				} 
				catch (RowNotFoundException $e) 
				{
					$error_controller = PHPBoostErrors::unexisting_page();
   					DispatchManager::redirect($error_controller);
				}
			}
			else
				$this->blog = new Blog();
		}
		return $this->blog;
	}
	
	private function check_pending_blog()
	{
		if (!$this->blog->is_published())
		{
			$this->tpl->put('NOT_VISIBLE_MESSAGE', MessageHelper::display(LangLoader::get_message('element.not_visible', 'status-messages-common'), MessageHelper::WARNING));
		}
		else
		{
			$this->blog->set_number_view($this->blog->get_number_view() + 1);
			BlogsService::update_number_view($this->blog);
		}
	}
	private function build_view(HTTPRequestCustom $request)
	{
		$current_page = $request->get_getint('page', 1);
		$config = BlogsConfig::load();
		
		$this->category = $this->blog->get_category();
		
		$blog_contents = $this->blog->get_contents();
		
		//If blog doesn't begin with a page, we insert one
		if (substr(trim($blog_contents), 0, 6) != '[page]')
		{
			$blog_contents = '[page]&nbsp;[/page]' . $blog_contents;
		}
		
		//Removing [page] bbcode
		$blog_contents_clean = preg_split('`\[page\].+\[/page\](.*)`Us', $blog_contents, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
		
		//Retrieving pages 
		preg_match_all('`\[page\]([^[]+)\[/page\]`U', $blog_contents, $array_page);

		$nbr_pages = count($array_page[1]);
		
		if ($nbr_pages > 1)
			$this->build_form($array_page, $current_page);
		
		$this->build_sources_view();
		
		$this->build_keywords_view();
		
		$page_name = (isset($array_page[1][$current_page-1]) && $array_page[1][$current_page-1] != '&nbsp;') ? $array_page[1][($current_page-1)] : '';
		
		$this->tpl->put_all(array_merge($this->blog->get_tpl_vars(), array(
			'C_COMMENTS_ENABLED' => $config->are_comments_enabled(),
			'C_NOTATION_ENABLED' => $config->is_notation_enabled(),
			'KERNEL_NOTATION' => NotationService::display_active_image($this->blog->get_notation()),
			'CONTENTS' => isset($blog_contents_clean[$current_page-1]) ? FormatingHelper::second_parse($blog_contents_clean[$current_page-1]) : '',
			'PAGE_NAME' => $page_name,
			'U_EDIT_BLOG' => $page_name !== '' ? BlogsUrlBuilder::edit_article($this->blog->get_id(), $current_page)->rel() : BlogsUrlBuilder::edit_article($this->blog->get_id())->rel(),
		)));
		
		$this->build_pages_pagination($current_page, $nbr_pages, $array_page);
		
		//Affichage commentaires
		if ($config->are_comments_enabled())
		{
			$comments_topic = new BlogsCommentsTopic($this->blog);
			$comments_topic->set_id_in_module($this->blog->get_id());
			$comments_topic->set_url(BlogsUrlBuilder::display_article($this->category->get_id(), $this->category->get_rewrited_name(), $this->blog->get_id(), $this->blog->get_rewrited_title()));

			$this->tpl->put('COMMENTS', $comments_topic->display());
		}
	}
	
	private function build_form($array_page, $current_page)
	{
		$form = new HTMLForm(__CLASS__, '', false);
		$form->set_css_class('options');
		
		$fieldset = new FormFieldsetHorizontal('pages', array('description' => $this->lang['blogs.summary']));
		
		$form->add_fieldset($fieldset);
		
		$blog_pages = $this->list_blog_pages($array_page);
		
		$fieldset->add_field(new FormFieldSimpleSelectChoice('blog_pages', '', $current_page, $blog_pages,
			array('class' => 'summary', 'events' => array('change' => 'document.location = "' . BlogsUrlBuilder::display_article($this->category->get_id(), $this->category->get_rewrited_name(), $this->blog->get_id(), $this->blog->get_rewrited_title())->rel() . '" + HTMLForms.getField("blog_pages").getValue();'))
		));
		
		$this->tpl->put('FORM', $form->display());
	}
	
	private function build_pages_pagination($current_page, $nbr_pages, $array_page)
	{
		if ($nbr_pages > 1)
		{	
			$pagination = $this->get_pagination($nbr_pages, $current_page);
			
			if ($current_page > 1 && $current_page <= $nbr_pages)
			{
				$previous_page = BlogsUrlBuilder::display_article($this->category->get_id(), $this->category->get_rewrited_name(), $this->blog->get_id(), $this->blog->get_rewrited_title())->rel() . ($current_page - 1);
				
				$this->tpl->put_all(array(
					'U_PREVIOUS_PAGE' => $previous_page,
					'L_PREVIOUS_TITLE' => $array_page[1][$current_page-2]
				));
			}

			if ($current_page > 0 && $current_page < $nbr_pages)
			{
				$next_page = BlogsUrlBuilder::display_article($this->category->get_id(), $this->category->get_rewrited_name(), $this->blog->get_id(), $this->blog->get_rewrited_title())->rel() . ($current_page + 1);
				
				$this->tpl->put_all(array(
					'U_NEXT_PAGE' => $next_page,
					'L_NEXT_TITLE' => $array_page[1][$current_page]
				));
			}
			
			$this->tpl->put_all(array(
				'C_PAGINATION' => true,
				'C_PREVIOUS_PAGE' => ($current_page != 1) ? true : false,
				'C_NEXT_PAGE' => ($current_page != $nbr_pages) ? true : false,
				'PAGINATION_BLOGS' => $pagination->display()
			));
		}
	}
	
	private function list_blog_pages($array_page)
	{
		$options = array();
		
		$i = 1;
		foreach ($array_page[1] as $page_name)
		{
			$options[] = new FormFieldSelectChoiceOption($page_name, $i++);
		}
		
		return $options;
	}
	
	private function build_sources_view()
	{
		$sources = $this->blog->get_sources();
		$nbr_sources = count($sources);
		$this->tpl->put('C_SOURCES', $nbr_sources > 0);
		
		$i = 1;
		foreach ($sources as $name => $url)
		{
			$this->tpl->assign_block_vars('sources', array(
				'C_SEPARATOR' => $i < $nbr_sources,
				'NAME' => $name,
				'URL' => $url,
			));
			$i++;
		}
	}
	
	private function build_keywords_view()
	{
		$keywords = $this->blog->get_keywords();
		$nbr_keywords = count($keywords);
		$this->tpl->put('C_KEYWORDS', $nbr_keywords > 0);
		
		$i = 1;
		foreach ($keywords as $keyword)
		{
			$this->tpl->assign_block_vars('keywords', array(
				'C_SEPARATOR' => $i < $nbr_keywords,
				'NAME' => $keyword->get_name(),
				'URL' => BlogsUrlBuilder::display_tag($keyword->get_rewrited_name())->rel(),
			));
			$i++;
		}
	}
	
	private function check_authorizations()
	{
		$blog = $this->get_blog();
		
		$auth_write = BlogsAuthorizationsService::check_authorizations($blog->get_id_category())->write();
		$auth_moderation = BlogsAuthorizationsService::check_authorizations($blog->get_id_category())->moderation();
		
		$not_authorized = !$auth_moderation && (!$auth_write && $blog->get_author_user()->get_id() != AppContext::get_current_user()->get_id());
		
		switch ($blog->get_publishing_state()) 
		{
			case Blog::PUBLISHED_NOW:
				if (!BlogsAuthorizationsService::check_authorizations($blog->get_id_category())->read() && $not_authorized)
				{
					$error_controller = PHPBoostErrors::user_not_authorized();
		   			DispatchManager::redirect($error_controller);
				}
			break;
			case Blog::NOT_PUBLISHED:
				if ($not_authorized)
				{
					$error_controller = PHPBoostErrors::user_not_authorized();
		   			DispatchManager::redirect($error_controller);
				}
			break;
			case Blog::PUBLISHED_DATE:
				if (!$blog->is_published() && $not_authorized)
				{
					$error_controller = PHPBoostErrors::user_not_authorized();
		   			DispatchManager::redirect($error_controller);
				}
			break;
			default:
				$error_controller = PHPBoostErrors::unexisting_page();
				DispatchManager::redirect($error_controller);
			break;
		}
	}
	
	private function get_pagination($nbr_pages, $current_page)
	{
		$pagination = new ModulePagination($current_page, $nbr_pages, 1, Pagination::LIGHT_PAGINATION);
		$pagination->set_url(BlogsUrlBuilder::display_article($this->category->get_id(), $this->category->get_rewrited_name(), $this->blog->get_id(), $this->blog->get_rewrited_title(), '%d'));
		
		if ($pagination->current_page_is_empty() && $current_page > 1)
		{
			$error_controller = PHPBoostErrors::unexisting_page();
			DispatchManager::redirect($error_controller);
		}
		
		return $pagination;
	}
	
	private function generate_response()
	{
		$response = new SiteDisplayResponse($this->tpl);

		$graphical_environment = $response->get_graphical_environment();
		$graphical_environment->set_page_title($this->blog->get_title(), $this->lang['blogs']);
		$graphical_environment->get_seo_meta_data()->set_description($this->blog->get_description());
		$graphical_environment->get_seo_meta_data()->set_canonical_url(BlogsUrlBuilder::display_article($this->category->get_id(), $this->category->get_rewrited_name(), $this->blog->get_id(), $this->blog->get_rewrited_title(), AppContext::get_request()->get_getint('page', 1)));
	
		$breadcrumb = $graphical_environment->get_breadcrumb();
		$breadcrumb->add($this->lang['blogs'], BlogsUrlBuilder::home());
		
		$categories = array_reverse(BlogsService::get_categories_manager()->get_parents($this->blog->get_id_category(), true));
		foreach ($categories as $id => $category)
		{
			if ($category->get_id() != Category::ROOT_CATEGORY)
				$breadcrumb->add($category->get_name(), BlogsUrlBuilder::display_category($category->get_id(), $category->get_rewrited_name()));
		}
		$breadcrumb->add($this->blog->get_title(), BlogsUrlBuilder::display_article($category->get_id(), $category->get_rewrited_name(), $this->blog->get_id(), $this->blog->get_rewrited_title()));
		
		return $response;
	}
}
?>
