<?php
/*##################################################
 *                       BlogsFormController.class.php
 *                            -------------------
 *   begin                : February 27, 2013
 *   copyright            : (C) 2013 Patrick DUBEAU
 *   email                : daaxwizeman@gmail.com
 *
 *
 ###################################################
 *		Module Articles : Dupliqué et modifié.
 *--------------------------------------------------
 * Date: 31.05.2016
 * Par: Swan | EDN
 * Site: http://easy-design.net
 * Contact : contact@easy-design.net
 *
 ###################################################
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 ###################################################*/

/**
 * @author Patrick DUBEAU <daaxwizeman@gmail.com>
 */
class BlogsFormController extends ModuleController
{
	/**
	 * @var HTMLForm
	 */
	private $form;
	/**
	 * @var FormButtonSubmit
	 */
	private $submit_button;
	
	private $tpl;
	
	private $lang;
	private $common_lang;
	
	private $blog;
	private $is_new_blog;

	public function execute(HTTPRequestCustom $request)
	{
		$this->init();
		$this->check_authorizations();
		$this->build_form($request);
		
		if ($this->submit_button->has_been_submited() && $this->form->validate())
		{
			$this->save();
			$this->redirect();
		}
		
		$this->tpl->put_all(array(
			'FORM' => $this->form->display(),
			'C_TINYMCE_EDITOR' => AppContext::get_current_user()->get_editor() == 'TinyMCE'
		));
		
		return $this->build_response($this->tpl);
	}

	private function init()
	{
		$this->lang = LangLoader::get('common', 'blogs');
		$this->tpl = new FileTemplate('blogs/BlogsFormController.tpl');
		$this->tpl->add_lang($this->lang);
		$this->common_lang = LangLoader::get('common');
	}
	
	private function build_form(HTTPRequestCustom $request)
	{
		$form = new HTMLForm(__CLASS__);
		
		$fieldset = new FormFieldsetHTML('blogs', $this->lang['blogs']);
		$form->add_fieldset($fieldset);

		$fieldset->add_field(new FormFieldTextEditor('title', $this->common_lang['form.title'], $this->get_blog()->get_title(), 
			array('required' => true)
		));

		if (!$this->is_contributor_member())
		{
			$fieldset->add_field(new FormFieldCheckbox('personalize_rewrited_title', $this->common_lang['form.rewrited_name.personalize'], $this->get_blog()->rewrited_title_is_personalized(),
				array('events' => array('click' =>'
					if (HTMLForms.getField("personalize_rewrited_title").getValue()) {
						HTMLForms.getField("rewrited_title").enable();
					} else { 
						HTMLForms.getField("rewrited_title").disable();
					}'
				))
			));

			$fieldset->add_field(new FormFieldTextEditor('rewrited_title', $this->common_lang['form.rewrited_name'], $this->get_blog()->get_rewrited_title(),
				array('description' => $this->common_lang['form.rewrited_name.description'],
				      'hidden' => !$this->get_blog()->rewrited_title_is_personalized()),
				array(new FormFieldConstraintRegex('`^[a-z0-9\-]+$`i'))
			));
		}

		$search_category_children_options = new SearchCategoryChildrensOptions();
		$search_category_children_options->add_authorizations_bits(Category::READ_AUTHORIZATIONS);
		$search_category_children_options->add_authorizations_bits(Category::CONTRIBUTION_AUTHORIZATIONS);
		$fieldset->add_field(BlogsService::get_categories_manager()->get_select_categories_form_field('id_category', $this->common_lang['form.category'], $this->get_blog()->get_id_category(), $search_category_children_options));
		
		$fieldset->add_field(new FormFieldCheckbox('enable_description', $this->lang['blogs.form.description_enabled'], $this->get_blog()->get_description_enabled(), 
			array('description' => StringVars::replace_vars($this->lang['blogs.form.description_enabled.description'], 
			array('number' => BlogsConfig::load()->get_number_character_to_cut())), 
				'events' => array('click' => '
					if (HTMLForms.getField("enable_description").getValue()) {
						HTMLForms.getField("description").enable();
					} else { 
						HTMLForms.getField("description").disable();
					}'
		))));
		
		$fieldset->add_field(new FormFieldRichTextEditor('description', StringVars::replace_vars($this->lang['blogs.form.description'],
			array('number' =>BlogsConfig::load()->get_number_character_to_cut())), $this->get_blog()->get_description(),
			array('rows' => 3, 'hidden' => !$this->get_blog()->get_description_enabled())
		));
		
		$fieldset->add_field(new FormFieldRichTextEditor('contents', $this->common_lang['form.contents'], $this->get_blog()->get_contents(),
			array('rows' => 15, 'required' => true)
		));
		
		$fieldset->add_field(new FormFieldActionLink('add_page', $this->lang['blogs.form.add_page'] , 'javascript:bbcode_page();', 'fa-pagebreak'));
		
		$other_fieldset = new FormFieldsetHTML('other', $this->common_lang['form.other']);
		$form->add_fieldset($other_fieldset);

		$other_fieldset->add_field(new FormFieldCheckbox('author_name_displayed', LangLoader::get_message('config.author_displayed', 'admin-common'), $this->get_blog()->get_author_name_displayed()));

		$other_fieldset->add_field(new FormFieldCheckbox('notation_enabled', LangLoader::get_message('config.notation_enabled', 'admin-common'), $this->get_blog()->get_notation_enabled()));
		
		$other_fieldset->add_field(new FormFieldUploadPictureFile('picture', $this->common_lang['form.picture'], $this->get_blog()->get_picture()->relative()));

		$other_fieldset->add_field(BlogsService::get_keywords_manager()->get_form_field($this->get_blog()->get_id(), 'keywords', $this->common_lang['form.keywords'],  
			array('description' => $this->common_lang['form.keywords.description'])
		));

		$other_fieldset->add_field(new BlogsFormFieldSelectSources('sources', $this->common_lang['form.sources'], $this->get_blog()->get_sources()));

		if (!$this->is_contributor_member())
		{
			$publication_fieldset = new FormFieldsetHTML('publication', $this->common_lang['form.approbation']);
			$form->add_fieldset($publication_fieldset);

			$publication_fieldset->add_field(new FormFieldDateTime('date_created', $this->common_lang['form.date.creation'], $this->get_blog()->get_date_created(),
				array('required' => true)
			));

			$publication_fieldset->add_field(new FormFieldSimpleSelectChoice('publishing_state', $this->common_lang['form.approbation'], $this->get_blog()->get_publishing_state(),
				array(
					new FormFieldSelectChoiceOption($this->common_lang['form.approbation.not'], Blog::NOT_PUBLISHED),
					new FormFieldSelectChoiceOption($this->common_lang['form.approbation.now'], Blog::PUBLISHED_NOW),
					new FormFieldSelectChoiceOption($this->common_lang['status.approved.date'], Blog::PUBLISHED_DATE),
				),
				array('events' => array('change' => '
				if (HTMLForms.getField("publishing_state").getValue() == 2) {
					jQuery("#' . __CLASS__ . '_publishing_start_date_field").show();
					HTMLForms.getField("end_date_enable").enable();
				} else { 
					jQuery("#' . __CLASS__ . '_publishing_start_date_field").hide();
					HTMLForms.getField("end_date_enable").disable();
				}'))
			));

			$publication_fieldset->add_field(new FormFieldDateTime('publishing_start_date', $this->common_lang['form.date.start'], 
				($this->get_blog()->get_publishing_start_date() === null ? new Date() : $this->get_blog()->get_publishing_start_date()), 
				array('hidden' => ($this->get_blog()->get_publishing_state() != Blog::PUBLISHED_DATE))
			));

			$publication_fieldset->add_field(new FormFieldCheckbox('end_date_enable', $this->common_lang['form.date.end.enable'], $this->get_blog()->end_date_enabled(), 
				array('hidden' => ($this->get_blog()->get_publishing_state() != Blog::PUBLISHED_DATE),
					'events' => array('click' => '
						if (HTMLForms.getField("end_date_enable").getValue()) {
							HTMLForms.getField("publishing_end_date").enable();
						} else { 
							HTMLForms.getField("publishing_end_date").disable();
						}'
				))
			));

			$publication_fieldset->add_field(new FormFieldDateTime('publishing_end_date', $this->common_lang['form.date.end'], 
				($this->get_blog()->get_publishing_end_date() === null ? new date() : $this->get_blog()->get_publishing_end_date()), 
				array('hidden' => !$this->get_blog()->end_date_enabled())
			));
		}

		$this->build_contribution_fieldset($form);
		
		$fieldset->add_field(new FormFieldHidden('referrer', $request->get_url_referrer()));
		
		$this->submit_button = new FormButtonDefaultSubmit();
		$form->add_button($this->submit_button);
		$form->add_button(new FormButtonReset());
		
		$this->form = $form;
		
		// Positionnement à la bonne page quand on édite un blog avec plusieurs pages
		if ($this->get_blog()->get_id() !== null)
		{
			$current_page = $request->get_getstring('page', '');
			
			$this->tpl->put('C_PAGE', !empty($current_page));
			
			if (!empty($current_page))
			{
				$blog_contents = $this->blog->get_contents();
				
				//If blog doesn't begin with a page, we insert one
				if (substr(trim($blog_contents), 0, 6) != '[page]')
				{
					$blog_contents = '[page]&nbsp;[/page]' . $blog_contents;
				}
				
				//Removing [page] bbcode
				$blog_contents_clean = preg_split('`\[page\].+\[/page\](.*)`Us', $blog_contents, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
				
				//Retrieving pages 
				preg_match_all('`\[page\]([^[]+)\[/page\]`U', $blog_contents, $array_page);

				$page_name = (isset($array_page[1][$current_page-1]) && $array_page[1][$current_page-1] != '&nbsp;') ? $array_page[1][($current_page-1)] : '';
				
				$this->tpl->put('PAGE', TextHelper::to_js_string($page_name));
			}
		}
	}

	private function build_contribution_fieldset($form)
	{
		if ($this->get_blog()->get_id() === null && $this->is_contributor_member())
		{
			$fieldset = new FormFieldsetHTML('contribution', LangLoader::get_message('contribution', 'user-common'));
			$fieldset->set_description(MessageHelper::display(LangLoader::get_message('contribution.explain', 'user-common'), MessageHelper::WARNING)->render());
			$form->add_fieldset($fieldset);

			$fieldset->add_field(new FormFieldRichTextEditor('contribution_description', LangLoader::get_message('contribution.description', 'user-common'), '', 
				array('description' => LangLoader::get_message('contribution.description.explain', 'user-common'))));
		}
	}

	private function is_contributor_member()
	{
		return (!BlogsAuthorizationsService::check_authorizations()->write() && BlogsAuthorizationsService::check_authorizations()->contribution());
	}

	private function get_blog()
	{
		if ($this->blog === null)
		{
			$id = AppContext::get_request()->get_getint('id', 0);
			if (!empty($id))
			{
				try
				{
					$this->blog = BlogsService::get_blog('WHERE blogs.id=:id', array('id' => $id));
				}
				catch(RowNotFoundException $e)
				{
					$error_controller = PHPBoostErrors::unexisting_page();
					DispatchManager::redirect($error_controller);
				}
			}
			else
			{
				$this->is_new_blog = true;
				$this->blog = new Blog();
				$this->blog->init_default_properties(AppContext::get_request()->get_getint('id_category', Category::ROOT_CATEGORY));
			}
		}
		return $this->blog;
	}

	private function check_authorizations()
	{
		$blog = $this->get_blog();
                
		if ($blog->get_id() === null)
		{
			if (!$blog->is_authorized_to_add())
			{
				$error_controller = PHPBoostErrors::user_not_authorized();
				DispatchManager::redirect($error_controller);
			}
		}
		else
		{
			if (!$blog->is_authorized_to_edit())
			{
				$error_controller = PHPBoostErrors::user_not_authorized();
				DispatchManager::redirect($error_controller);
			}
		}
		if (AppContext::get_current_user()->is_readonly())
		{
			$controller = PHPBoostErrors::user_in_read_only();
			DispatchManager::redirect($controller);
		}
	}

	private function save()
	{
		$blog = $this->get_blog();
		
		$blog->set_title($this->form->get_value('title'));
		$blog->set_id_category($this->form->get_value('id_category')->get_raw_value());
		$blog->set_description(($this->form->get_value('enable_description') ? $this->form->get_value('description') : ''));
		$blog->set_contents($this->form->get_value('contents'));
		
		$author_name_displayed = $this->form->get_value('author_name_displayed') ? $this->form->get_value('author_name_displayed') : Blog::AUTHOR_NAME_NOTDISPLAYED;
		$blog->set_author_name_displayed($author_name_displayed);
		$notation_enabled = $this->form->get_value('notation_enabled') ? $this->form->get_value('notation_enabled') : Blog::NOTATION_DISABLED;
		$blog->set_notation_enabled($notation_enabled);
		$blog->set_picture(new Url($this->form->get_value('picture')));
		
		$blog->set_sources($this->form->get_value('sources'));
		
		if ($this->is_contributor_member())
		{
			if ($blog->get_id() === null)
				$blog->set_date_created(new Date());
			
			$blog->set_rewrited_title(Url::encode_rewrite($blog->get_title()));
			$blog->set_publishing_state(Blog::NOT_PUBLISHED);
			$blog->clean_publishing_start_and_end_date();
		}
		else
		{
			$blog->set_date_created($this->form->get_value('date_created'));
			$rewrited_title = $this->form->get_value('rewrited_title', '');
			$rewrited_title = $this->form->get_value('personalize_rewrited_title') && !empty($rewrited_title) ? $rewrited_title : Url::encode_rewrite($blog->get_title());
			$blog->set_rewrited_title($rewrited_title);

			$blog->set_publishing_state($this->form->get_value('publishing_state')->get_raw_value());
			if ($blog->get_publishing_state() == Blog::PUBLISHED_DATE)
			{
				$config = BlogsConfig::load();
				$deferred_operations = $config->get_deferred_operations();
				
				$old_start_date = $blog->get_publishing_start_date();
				$start_date = $this->form->get_value('publishing_start_date');
				$blog->set_publishing_start_date($start_date);
				
				if ($old_start_date !== null && $old_start_date->get_timestamp() != $start_date->get_timestamp() && in_array($old_start_date->get_timestamp(), $deferred_operations))
				{
					$key = array_search($old_start_date->get_timestamp(), $deferred_operations);
					unset($deferred_operations[$key]);
				}
				
				if (!in_array($start_date->get_timestamp(), $deferred_operations))
					$deferred_operations[] = $start_date->get_timestamp();
				
				if ($this->form->get_value('end_date_enable'))
				{
					$old_end_date = $blog->get_publishing_end_date();
					$end_date = $this->form->get_value('publishing_end_date');
					$blog->set_publishing_end_date($end_date);
					
					if ($old_end_date !== null && $old_end_date->get_timestamp() != $end_date->get_timestamp() && in_array($old_end_date->get_timestamp(), $deferred_operations))
					{
						$key = array_search($old_end_date->get_timestamp(), $deferred_operations);
						unset($deferred_operations[$key]);
					}
					
					if (!in_array($end_date->get_timestamp(), $deferred_operations))
						$deferred_operations[] = $end_date->get_timestamp();
				}
				else
				{
					$blog->clean_publishing_end_date();
				}
				
				$config->set_deferred_operations($deferred_operations);
				BlogsConfig::save();
			}
			else
			{
				$blog->clean_publishing_start_and_end_date();
			}
		}
		
		if ($blog->get_id() === null)
		{
			$blog->set_author_user(AppContext::get_current_user());
			$id_blog = BlogsService::add($blog);
		}
		else
		{
			$now = new Date();
			$blog->set_date_updated($now);
			$id_blog = $blog->get_id();
			BlogsService::update($blog);
		}

		$this->contribution_actions($blog, $id_blog);
		
		BlogsService::get_keywords_manager()->put_relations($id_blog, $this->form->get_value('keywords'));

		Feed::clear_cache('blogs');
		BlogsCategoriesCache::invalidate();
	}

	private function contribution_actions(Blog $blog, $id_blog)
	{
		if ($blog->get_id() === null)
		{
			if ($this->is_contributor_member())
			{
				$contribution = new Contribution();
				$contribution->set_id_in_module($id_blog);
				$contribution->set_description(stripslashes($this->form->get_value('contribution_description')));
				$contribution->set_entitled($blog->get_title());
				$contribution->set_fixing_url(BlogsUrlBuilder::edit_article($id_blog)->relative());
				$contribution->set_poster_id(AppContext::get_current_user()->get_id());
				$contribution->set_module('blogs');
				$contribution->set_auth(
					Authorizations::capture_and_shift_bit_auth(
						BlogsService::get_categories_manager()->get_heritated_authorizations($blog->get_id_category(), Category::MODERATION_AUTHORIZATIONS, Authorizations::AUTH_CHILD_PRIORITY),
						Category::MODERATION_AUTHORIZATIONS, Contribution::CONTRIBUTION_AUTH_BIT
					)
				);
				ContributionService::save_contribution($contribution);
			}
		}
		else
		{
			$corresponding_contributions = ContributionService::find_by_criteria('blogs', $id_blog);
			if (count($corresponding_contributions) > 0)
			{
				$blog_contribution = $corresponding_contributions[0];
				$blog_contribution->set_status(Event::EVENT_STATUS_PROCESSED);

				ContributionService::save_contribution($blog_contribution);
			}
		}
		$blog->set_id($id_blog);
	}
	
	private function redirect()
	{
		$blog = $this->get_blog();
		$category = $blog->get_category();

		if ($this->is_new_blog && $this->is_contributor_member() && !$blog->is_published())
		{
			DispatchManager::redirect(new UserContributionSuccessController());
		}
		elseif ($blog->is_published())
		{
			if ($this->is_new_blog)
				AppContext::get_response()->redirect(BlogsUrlBuilder::display_article($category->get_id(), $category->get_rewrited_name(), $blog->get_id(), $blog->get_rewrited_title(), AppContext::get_request()->get_getint('page', 1)), StringVars::replace_vars($this->lang['blogs.message.success.add'], array('title' => $blog->get_title())));
			else
				AppContext::get_response()->redirect(($this->form->get_value('referrer') ? $this->form->get_value('referrer') : BlogsUrlBuilder::display_article($category->get_id(), $category->get_rewrited_name(), $blog->get_id(), $blog->get_rewrited_title(), AppContext::get_request()->get_getint('page', 1))), StringVars::replace_vars($this->lang['blogs.message.success.edit'], array('title' => $blog->get_title())));
		}
		else
		{
			if ($this->is_new_blog)
				AppContext::get_response()->redirect(BlogsUrlBuilder::display_pending_blogs(), StringVars::replace_vars($this->lang['blogs.message.success.add'], array('title' => $blog->get_title())));
			else
				AppContext::get_response()->redirect(($this->form->get_value('referrer') ? $this->form->get_value('referrer') : BlogsUrlBuilder::display_pending_blogs()), StringVars::replace_vars($this->lang['blogs.message.success.edit'], array('title' => $blog->get_title())));
		}
	}

	private function build_response(View $tpl)
	{
		$blog = $this->get_blog();
		
		$response = new SiteDisplayResponse($tpl);
		$graphical_environment = $response->get_graphical_environment();
		
		$breadcrumb = $graphical_environment->get_breadcrumb();
		$breadcrumb->add($this->lang['blogs'], BlogsUrlBuilder::home());

		if ($blog->get_id() === null)
		{
			$breadcrumb->add($this->lang['blogs.add'], BlogsUrlBuilder::add_blog($blog->get_id_category()));
			$graphical_environment->set_page_title($this->lang['blogs.add'], $this->lang['blogs']);
			$graphical_environment->get_seo_meta_data()->set_description($this->lang['blogs.add']);
			$graphical_environment->get_seo_meta_data()->set_canonical_url(BlogsUrlBuilder::add_blog($blog->get_id_category()));
		}
		else
		{
			$categories = array_reverse(BlogsService::get_categories_manager()->get_parents($blog->get_id_category(), true));
			foreach ($categories as $id => $category)
			{
				if ($category->get_id() != Category::ROOT_CATEGORY)
					$breadcrumb->add($category->get_name(), BlogsUrlBuilder::display_category($category->get_id(), $category->get_rewrited_name()));
			}
			$breadcrumb->add($blog->get_title(), BlogsUrlBuilder::display_article($category->get_id(), $category->get_rewrited_name(), $blog->get_id(), $blog->get_rewrited_title()));

			$breadcrumb->add($this->lang['blogs.edit'], BlogsUrlBuilder::edit_article($blog->get_id()));
			$graphical_environment->set_page_title($this->lang['blogs.edit'], $this->lang['blogs']);
			$graphical_environment->get_seo_meta_data()->set_description($this->lang['blogs.edit']);
			$graphical_environment->get_seo_meta_data()->set_canonical_url(BlogsUrlBuilder::edit_article($blog->get_id()));
		}

		return $response;
	}
}
?>
