<?php
/*##################################################
 *		       BlogsPrintBlogsController.class.php
 *                            -------------------
 *   begin                : June 03, 2013
 *   copyright            : (C) 2013 Patrick DUBEAU
 *   email                : daaxwizeman@gmail.com
 *
 *
  ###################################################
 *		Module Articles : Dupliqu et modifi.
 *--------------------------------------------------
 * Date: 31.05.2016
 * Par: Swan | EDN
 * Site: http://easy-design.net
 * Contact : contact@easy-design.net
 *
 ###################################################
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 ###################################################*/

class BlogsPrintBlogsController extends ModuleController
{	
	private $lang;
	private $view;
	private $blog;
	
	public function execute(HTTPRequestCustom $request)
	{
		$this->check_authorizations();
		
		$this->init();
		
		$this->build_view($request);
		
		return new SiteNodisplayResponse($this->view);
	}
	
	private function init()
	{
		$this->lang = LangLoader::get('common', 'blogs');
		$this->view = new FileTemplate('framework/content/print.tpl');
		$this->view->add_lang($this->lang);
	}
	
	private function get_blog()
	{
		if ($this->blog === null)
		{
			$id = AppContext::get_request()->get_getint('id', 0);
			if (!empty($id))
			{
				try 
				{
					$this->blog = BlogsService::get_blog('WHERE blogs.id=:id', array('id' => $id));
				} 
				catch (RowNotFoundException $e) 
				{
					$error_controller = PHPBoostErrors::unexisting_page();
					DispatchManager::redirect($error_controller);
				}
			}
			else
				$this->blog = new Blog();
		}
		return $this->blog;
	}
	
	private function build_view()
	{
		$contents = preg_replace('`\[page\](.*)\[/page\]`', '<h2>$1</h2>', $this->blog->get_contents());
		$this->view->put_all(array(
			'PAGE_TITLE' => $this->lang['blogs.print.blog'] . ' - ' . $this->blog->get_title() . ' - ' . GeneralConfig::load()->get_site_name(),
			'TITLE' => $this->blog->get_title(),
			'CONTENT' => FormatingHelper::second_parse($contents)
		));
	}
	
	private function check_authorizations()
	{
		$blog = $this->get_blog();
		
		$not_authorized = !BlogsAuthorizationsService::check_authorizations($blog->get_id_category())->write() && (!BlogsAuthorizationsService::check_authorizations($blog->get_id_category())->moderation() && $blog->get_author_user()->get_id() != AppContext::get_current_user()->get_id());
		
		switch ($blog->get_publishing_state()) 
		{
			case Blog::PUBLISHED_NOW:
				if (!BlogsAuthorizationsService::check_authorizations()->read() && $not_authorized)
				{
					$error_controller = PHPBoostErrors::user_not_authorized();
					DispatchManager::redirect($error_controller);
				}
			break;
			case Blog::NOT_PUBLISHED:
				if ($not_authorized)
				{
					$error_controller = PHPBoostErrors::user_not_authorized();
					DispatchManager::redirect($error_controller);
				}
			break;
			case Blog::PUBLISHED_DATE:
				if (!$blog->is_published() && $not_authorized)
				{
					$error_controller = PHPBoostErrors::user_not_authorized();
					DispatchManager::redirect($error_controller);
				}
			break;
			default:
				$error_controller = PHPBoostErrors::unexisting_page();
				DispatchManager::redirect($error_controller);
			break;
		}
	}
}
?>